#include <couchdb-glib.h>

#include "couchdb-result.h"

#define GET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((o), COUCHDB_TYPE_RESULT, CouchdbResultPrivate))

G_DEFINE_TYPE (CouchdbResult, couchdb_result, G_TYPE_OBJECT);

struct _CouchdbResultPrivate {
	gboolean reduced;
	gsize total_rows;
	guint offset;

	GSList *documents;
	GHashTable *reduce_result;
};

enum {
	PROP_DUMMY,

	PROP_REDUCED,
	PROP_TOTAL_ROWS,
	PROP_OFFSET,
	PROP_DOCUMENTS,
	PROP_REDUCE_RESULT
};

static void
couchdb_result_dispose (GObject *object)
{
	CouchdbResult * self = COUCHDB_RESULT (object);

	if (!self->priv->reduced && self->priv->documents != NULL) {
		couchdb_database_free_document_list (self->priv->documents);
		self->priv->documents = NULL;
	}

	G_OBJECT_CLASS (couchdb_result_parent_class)->dispose (object);
}

static void
couchdb_result_finalize (GObject *object)
{
	CouchdbResult * self = COUCHDB_RESULT (object);

	if (self->priv->reduce_result) {
		g_hash_table_destroy (self->priv->reduce_result);
	}

	G_OBJECT_CLASS (couchdb_result_parent_class)->dispose (object);
}

static void
couchdb_result_init (CouchdbResult *self)
{
	self->priv = GET_PRIVATE (self);
}

static void
couchdb_result_set_property (GObject *object, guint prop_id,
				      const GValue *value, GParamSpec *pspec)
{
	CouchdbResult *self = COUCHDB_RESULT (object);

	switch (prop_id) {
	case PROP_REDUCED:
		self->priv->reduced = g_value_get_boolean (value);
		break;
	case PROP_TOTAL_ROWS:
		self->priv->total_rows = g_value_get_uint (value);
		break;
	case PROP_OFFSET:
		self->priv->offset = g_value_get_uint (value);
		break;
	case PROP_DOCUMENTS:
		if (self->priv->documents)
			couchdb_database_free_document_list (self->priv->documents);
		self->priv->documents = g_value_get_pointer (value);
		break;
	case PROP_REDUCE_RESULT:
		if (self->priv->reduce_result)
			g_hash_table_destroy (self->priv->reduce_result);
		self->priv->reduce_result = g_value_get_pointer (value);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
		break;
	}
}

static void
couchdb_result_get_property (GObject *object, guint prop_id,
				      GValue *value, GParamSpec *pspec)
{
	CouchdbResult *self = COUCHDB_RESULT (object);

	switch (prop_id) {
	case PROP_REDUCED:
		g_value_set_boolean (value, self->priv->reduced);
		break;
	case PROP_TOTAL_ROWS:
		g_value_set_uint64 (value, self->priv->total_rows);
		break;
	case PROP_OFFSET:
		g_value_set_uint (value, self->priv->offset);
		break;
	case PROP_DOCUMENTS:
		g_value_set_pointer (value, self->priv->documents);
		break;
	case PROP_REDUCE_RESULT:
		g_value_set_pointer (value, self->priv->reduce_result);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
		break;
	}
}

static void
couchdb_result_class_init (CouchdbResultClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);
	GParamSpec *pspec;

	object_class->dispose = couchdb_result_dispose;
	object_class->finalize = couchdb_result_finalize;
	object_class->get_property = couchdb_result_get_property;
	object_class->set_property = couchdb_result_set_property;

	g_type_class_add_private(klass, sizeof(CouchdbResultPrivate));

	pspec =
	  g_param_spec_boolean ("reduced", "Reduced",
				"Whether result was created from reduced view",
				FALSE,
				G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
	g_object_class_install_property (object_class, PROP_REDUCED, pspec);

	pspec =
	  g_param_spec_uint ("total-rows", "Total rows",
	  		     "A total number of rows in a view",
			     0, G_MAXUINT, 0,
			     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
	g_object_class_install_property (object_class, PROP_TOTAL_ROWS, pspec);

	pspec =
	  g_param_spec_uint ("offset", "Offset",
	  		     "An offset at which returned rows start",
			     0, G_MAXUINT, 0,
			     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
	g_object_class_install_property (object_class, PROP_OFFSET, pspec);

	pspec =
	  g_param_spec_pointer ("documents", "Documents",
	  			"A list of documents returned from a view",
				G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
	g_object_class_install_property (object_class, PROP_DOCUMENTS, pspec);

	pspec =
	  g_param_spec_pointer ("reduce-result", "Reduce result",
	  			"A result of reduce being run on a view",
				G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
	g_object_class_install_property (object_class, PROP_REDUCE_RESULT, pspec);
}

gboolean
couchdb_result_is_reduced (CouchdbResult *self)
{
	g_return_val_if_fail (COUCHDB_IS_RESULT (self), FALSE);

	return self->priv->reduced;
}

gsize
couchdb_result_get_total_rows (CouchdbResult *self)
{
	g_return_val_if_fail (COUCHDB_IS_RESULT (self), 0);

	return self->priv->total_rows;
}

guint
couchdb_result_get_offset (CouchdbResult *self)
{
	g_return_val_if_fail (COUCHDB_IS_RESULT (self), 0);

	return self->priv->offset;
}

const GSList *
couchdb_result_get_document_list (CouchdbResult *self)
{
	g_return_val_if_fail (COUCHDB_IS_RESULT (self), NULL);

	return self->priv->documents;
}

const GHashTable *
couchdb_result_get_reduce_result (CouchdbResult *self)
{
	g_return_val_if_fail (COUCHDB_IS_RESULT (self), 0);

	return self->priv->reduce_result;
}
